unit Edit;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, FileDlg;

type
  TEditForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    NewWindow1: TMenuItem;
    N7: TMenuItem;
    Open2: TMenuItem;
    New2: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    Saveas1: TMenuItem;
    N1: TMenuItem;
    Print1: TMenuItem;
    Printersetup1: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    ExitAll1: TMenuItem;
    Edit1: TMenuItem;
    Undo1: TMenuItem;
    N6: TMenuItem;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    Delete1: TMenuItem;
    N3: TMenuItem;
    Selectall1: TMenuItem;
    Character1: TMenuItem;
    Left1: TMenuItem;
    Right1: TMenuItem;
    Center1: TMenuItem;
    N4: TMenuItem;
    Wordwrap1: TMenuItem;
    N5: TMenuItem;
    Font1: TMenuItem;
    Window1: TMenuItem;
    Cascade1: TMenuItem;
    Tile1: TMenuItem;
    PopupMenu1: TPopupMenu;
    Cut2: TMenuItem;
    Copy2: TMenuItem;
    Paste2: TMenuItem;
    FileDialog: TFileDialog;
    FontDialog1: TFontDialog;
    PrintDialog1: TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    Editor: TMemo;
    procedure AlignClick(Sender: TObject);
    procedure Cascade1Click(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
    procedure Cut1Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure ExitAll1Click(Sender: TObject);
    procedure Font1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure New2Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Open2Click(Sender: TObject);
    procedure Paste1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure Printersetup1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure SelectAll1Click(Sender: TObject);
    procedure Tile1Click(Sender: TObject);
    procedure Undo1Click(Sender: TObject);
    procedure WordWrap1Click(Sender: TObject);
  private
    PathName: string;
  public
    procedure Open(const AFileName: string);
  end;

var
  EditForm: TEditForm;

const
  DefaultFileName = 'Unbenannt';

implementation

uses Clipbrd, Printers;
{$R *.DFM}

procedure TEditForm.AlignClick(Sender: TObject);
begin
  Left1.Checked := False;
  Right1.Checked := False;
  Center1.Checked := False;
  with Sender as TMenuItem do Checked := True;
  with Editor do
    if Left1.Checked then
      Alignment := taLeftJustify
    else if Right1.Checked then
      Alignment := taRightJustify
    else if Center1.Checked then
      Alignment := taCenter;
end;

procedure TEditForm.Cascade1Click(Sender: TObject);
var
  I, J, Step: Integer;
begin
  J := 0;
  with Application do
    for I := 0 to (ComponentCount - 1) do
      if Components[I] is TEditForm then
        with (Components[I] as TEditForm) do begin
          Inc(J);
          Step := Height - ClientHeight;
          Left := J * Step;
          Top := J * Step;
          Width := GetSystemMetrics(SM_CXSCREEN) div 2;
          Height := GetSystemMetrics(SM_CYSCREEN) div 2;
        end;
end;

procedure TEditForm.Copy1Click(Sender: TObject);
begin
  Editor.CopyToClipboard;
end;

procedure TEditForm.Cut1Click(Sender: TObject);
begin
  Editor.CutToClipboard;
end;

procedure TEditForm.Delete1Click(Sender: TObject);
begin
  Editor.ClearSelection;
end;

procedure TEditForm.Edit1Click(Sender: TObject);
var
  HasSelection: Boolean;
begin
  Paste1.Enabled := Clipboard.HasFormat(CF_TEXT);
  Paste2.Enabled := Paste1.Enabled;
  HasSelection := Editor.SelLength > 0;
  Undo1.Enabled := Boolean(SendMessage(Editor.Handle, EM_CANUNDO, 0, 0));
  Cut1.Enabled := HasSelection;
  Cut2.Enabled := HasSelection;
  Copy1.Enabled := HasSelection;
  Copy2.Enabled := HasSelection;
  Delete1.Enabled := HasSelection;
end;

procedure TEditForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TEditForm.ExitAll1Click(Sender: TObject);
var I: Integer;
begin
  with Application do
    for I := 0 to (ComponentCount - 1) do
      if Components[I] is TEditForm then
        (Components[I] as TEditForm).Close;
end;

procedure TEditForm.Font1Click(Sender: TObject);
begin
  FontDialog1.Font := Editor.Font;
  if FontDialog1.Execute then
    Editor.Font.Assign(FontDialog1.Font);
end;

procedure TEditForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TEditForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
const
  SWarningText = 'Mchten Sie die nderungen in %s speichern?';
begin
  if Editor.Modified then begin
    case MessageDlg(Format(SWarningText, [PathName]), mtConfirmation,
      [mbYes, mbNo, mbCancel], 0) of
      idYes: Save1Click(Self);
      idCancel: CanClose := False;
    end;
  end;
end;

procedure TEditForm.FormCreate(Sender: TObject);
begin
  PathName := DefaultFileName;
  Editor.Clear;
end;

procedure TEditForm.New1Click(Sender: TObject);
const
  SWarningText = 'Mchten Sie die nderungen in %s speichern?';
begin
  if Editor.Modified then begin
    case MessageDlg(Format(SWarningText, [PathName]), mtConfirmation,
                    [mbYes, mbNo, mbCancel], 0) of
      idYes: Save1Click(Self);
      idCancel: Exit;
    end;
  end;
  PathName := DefaultFileName;
  Caption := DefaultFileName;
  Editor.Clear;
end;

procedure TEditForm.New2Click(Sender: TObject);
begin
  with TEditForm.Create(Application) do
    Tile1Click(Self);
end;

procedure TEditForm.Open(const AFileName: string);
begin
  PathName := AFileName;
  Caption := ExtractFileName(AFileName);
  with Editor do begin
    Lines.LoadFromFile(PathName);
    SelStart := 0;
    Modified := False;
  end;
end;

procedure TEditForm.Open1Click(Sender: TObject);
const
  SWarningText = 'Mchten Sie die nderungen in %s speichern?';
begin
  if Editor.Modified then begin
    case MessageDlg(Format(SWarningText, [PathName]), mtConfirmation,
                    [mbYes, mbNo, mbCancel], 0) of
      idYes: Save1Click(Self);
      idCancel: Exit;
    end;
  end;
  FileDialog.DialogMode := dmOpen;
  if FileDialog.Execute then
    Open(Self.FileDialog.FileName);
end;

procedure TEditForm.Open2Click(Sender: TObject);
begin
  FileDialog.DialogMode := dmOpen;
  if FileDialog.Execute then
    with TEditForm.Create(Application) do begin
      Tile1Click(Self);
      Open(Self.FileDialog.FileName);
    end;
end;

procedure TEditForm.Paste1Click(Sender: TObject);
begin
  Editor.PasteFromClipboard;
end;

procedure TEditForm.Print1Click(Sender: TObject);
var
  I: Integer;
  PrintText: System.Text;
begin
  if PrintDialog1.Execute then begin
    AssignPrn(PrintText);
    Rewrite(PrintText);
    Printer.Canvas.Font := Editor.Font;
    for I := 0 to Editor.Lines.Count - 1 do
      Writeln(PrintText, Editor.Lines[I]);
    CloseFile(PrintText);
  end;
end;

procedure TEditForm.Printersetup1Click(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;

procedure TEditForm.Save1Click(Sender: TObject);
begin
  if PathName = DefaultFileName then
    SaveAs1Click(Sender)
  else begin
    Editor.Lines.SaveToFile(PathName);
    Editor.Modified := False;
  end;
end;

procedure TEditForm.SaveAs1Click(Sender: TObject);
begin
  FileDialog.DialogMode := dmSave;
  FileDialog.FileName := PathName;
  if FileDialog.Execute then begin
    PathName := FileDialog.FileName;
    Caption := ExtractFileName(PathName);
    Save1Click(Sender);
  end;
end;

procedure TEditForm.SelectAll1Click(Sender: TObject);
begin
  Editor.SelectAll;
end;

procedure TEditForm.Tile1Click(Sender: TObject);
var
  I, J, K: Integer;
begin
  J := 0;
  K := 0;
  with Application do begin
    for I := 0 to (ComponentCount - 1) do
      if Components[I] is TEditForm then Inc(J);
    for I := 0 to (ComponentCount - 1) do
      if Components[I] is TEditForm then
        with (Components[I] as TEditForm) do begin
          Inc(K);
          if J < 5 then Width := GetSystemMetrics(SM_CXSCREEN) div 2
                   else Width := GetSystemMetrics(SM_CXSCREEN) div 3;
          if J < 7 then Height := GetSystemMetrics(SM_CYSCREEN) div 2
                   else Height := GetSystemMetrics(SM_CYSCREEN) div 3;
          if J < 5 then begin
            if K in[1, 3] then Left := 0
                          else Left := Width;
            if K < 3 then Top := 0
                     else Top := Height;
          end
          else begin
            if K in [1, 4, 7] then Left := 0
              else if K in [2, 5, 8] then Left := Width
                else Left := Width + Width;
            if K < 4 then Top := 0
              else if K < 7 then Top := Height
                else Top := Height + Height;
          end;
        end;
  end;
end;

procedure TEditForm.Undo1Click(Sender: TObject);
begin
  SendMessage(Editor.Handle, EM_UNDO, 0, 0);
end;

procedure TEditForm.WordWrap1Click(Sender: TObject);
begin
  with Editor do begin
    WordWrap := not WordWrap; { toggle word wrapping }
    if WordWrap then
      ScrollBars := ssVertical
    else
      ScrollBars := ssBoth;
    WordWrap1.Checked := WordWrap; { set menu item check }
  end;
end;

end.
