unit PatternDlg;

interface

uses
  Windows, SysUtils, Classes, Controls, StdCtrls, ExtCtrls,
  Dialogs, Forms, BackPropNet, Spin, FileDlg;

type
  TPatternEdit = class(TForm)
    AddBtn: TButton;
    CancelBtn: TButton;
    DeleteAllBtn: TButton;
    DeleteBtn: TButton;
    InsertBtn: TButton;
    OKBtn: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    InputSpin: TSpinEdit;
    OutputSpin: TSpinEdit;
    InputBox: TListBox;
    OutputBox: TListBox;
    SelectBox: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Panel1: TPanel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    procedure AddBtnClick(Sender: TObject);
    procedure DeleteAllBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Edit1DblClick(Sender: TObject);
    procedure Edit1KeyDown(Sender: TObject; var Key: Word;
                           Shift: TShiftState);                         
    procedure InputBoxClick(Sender: TObject);
    procedure InputSpinChange(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure OutputBoxClick(Sender: TObject);
    procedure OutputSpinChange(Sender: TObject);
    procedure SelectBoxChange(Sender: TObject);
  private
    ListBox: TListBox;
    PatternSet: TSet;
  public
    constructor CreateWithParam(AOwner: TComponent; ANet: TBackPropNet;
                            ASet: TSet; SetType: TSetType; const ATitle: string);
    procedure ReadSet(Index: Integer); virtual;
  end;

var
  PatternEdit: TPatternEdit;

implementation
{$R *.DFM}

constructor TPatternEdit.CreateWithParam(AOwner: TComponent; ANet: TBackPropNet;
                            ASet: TSet; SetType: TSetType; const ATitle: string);
begin
  inherited Create(AOwner);
  Caption := ATitle;
  if SetType = stRecallSet then begin
    Label2.Caption := 'A&usgangsvektor:';
    OutputBox.Enabled := False;
  end;
  PatternSet := ASet;
  ReadSet(0);
end;

procedure TPatternEdit.AddBtnClick(Sender: TObject);
begin
  with PatternSet do begin
    PatternNo := PatternNo + 1;
    ReadSet(PatternNo - 1);
  end;
end;

procedure TPatternEdit.DeleteAllBtnClick(Sender: TObject);
begin
  PatternSet.DeleteAll;
  ReadSet(0);
end;

procedure TPatternEdit.DeleteBtnClick(Sender: TObject);
var I: Integer;
begin
  I := SelectBox.ItemIndex;
  with PatternSet do begin
    Delete(I);
    if I = PatternNo then Dec(I);
    ReadSet(I);
  end;
end;

procedure TPatternEdit.Edit1Change(Sender: TObject);
var I: Integer; Val: Extended;
begin
  if (Edit1.Text = '') or (PatternSet.PatternNo = 0) then Exit;
  I := ListBox.ItemIndex;
  try
    Val := StrToFloat(Edit1.Text);
  except
    on EConvertError do begin
      MessageDlg('''' + Edit1.Text + '''' + ' ist kein gltiger Integerwert.', mtError, [mbOK], 0);
      Edit1.Text := ListBox.Items[I];
      Exit;
    end;
  end;
  ListBox.Items[I] := FloatToStrF(Val, ffExponent, 7, 2);
  ListBox.ItemIndex := I;
  case ListBox.Tag of
    1: PatternSet[SelectBox.ItemIndex].Inputs[I] := Val;
    2: PatternSet[SelectBox.ItemIndex].Outputs[I] := Val;
  end;
end;

procedure TPatternEdit.Edit1DblClick(Sender: TObject);
var Key: Word;
begin
  Key := VK_Return;
  Edit1KeyDown(Self, Key, [ssShift]);
end;

procedure TPatternEdit.Edit1KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  with ListBox do begin
    case Key of
      VK_Return, VK_Down: begin
        if ItemIndex = (Items.Count - 1) then
          if (ListBox.Tag = 1) and (OutputBox.Enabled) then begin
            OutputBoxClick(Self);
            Exit;
          end
          else if (ListBox.Tag = 1) and not(OutputBox.Enabled) then begin
            ItemIndex := -1;
            if Items.Count = 0 then Exit;
          end
          else begin
            InputBoxClick(Self);
            Exit;
          end;
          ItemIndex := ItemIndex + 1;
          Edit1.Text := Items[ItemIndex];
          Edit1.SelectAll;
          ActiveControl := Edit1;
      end;
      VK_Up: begin
        if ItemIndex = 0 then
          if (ListBox.Tag = 1) and (OutputBox.Enabled) then begin
            OutputBoxClick(Edit1);
            Exit;
          end
          else if (ListBox.Tag = 1) and not(OutputBox.Enabled) then begin
            ItemIndex := Items.Count;
            if Items.Count = 0 then Exit;
          end
          else begin
            InputBoxClick(Edit1);
            Exit;
          end;
          ItemIndex := ItemIndex - 1;
          Edit1.Text := Items[ItemIndex];
          ActiveControl := Edit1;
          Edit1.SelectAll;          
      end;
    end;
  end;
end;

procedure TPatternEdit.InputBoxClick(Sender: TObject);
begin
  OutputBox.ItemIndex := -1;
  with InputBox do
    if Items.Count > 0 then begin
      ListBox := InputBox;
      if ItemIndex = -1 then
        if Sender is TPatternEdit then ItemIndex := 0
                                  else ItemIndex := Items.Count - 1;
      Edit1.Text := Items[ItemIndex];
      Edit1.SelectAll;
    end
    else Edit1.Text := '';
    ActiveControl := Edit1;
end;

procedure TPatternEdit.InputSpinChange(Sender: TObject);
begin
  with InputSpin do begin
    if Text = '' then Exit;
    if PatternSet.InputNo <> Value then begin
      PatternSet.InputNo := Value;
      ReadSet(SelectBox.ItemIndex);
    end;
  end;
end;

procedure TPatternEdit.InsertBtnClick(Sender: TObject);
var Pattern: TPattern;
begin
  with PatternSet do begin
    Pattern := TPattern.Create(InputNo, OutputNo);
    try
      Insert(SelectBox.ItemIndex, Pattern);
    finally
      Pattern.Free;
    end;
    ReadSet(SelectBox.ItemIndex);
  end;
end;

procedure TPatternEdit.OutputBoxClick(Sender: TObject);
begin
  if (OutputBox.Enabled = False) then begin
    InputBoxClick(Self);
    Exit;
  end;
  InputBox.ItemIndex := -1;
  with OutputBox do
    if Items.Count > 0 then begin
      ListBox := OutputBox;
      if Sender is TPatternEdit then ItemIndex := 0
                                else ItemIndex := Items.Count - 1;
      Edit1.Text := Items[ItemIndex];
      Edit1.SelectAll;
    end
    else Edit1.Text := '';
  ActiveControl := Edit1;
end;

procedure TPatternEdit.OutputSpinChange(Sender: TObject);
begin
  with OutputSpin do begin
    if Text = '' then Exit;
    if PatternSet.OutputNo <> Value then begin
      PatternSet.OutputNo := Value;
      ReadSet(SelectBox.ItemIndex);
    end;
  end;
end;

procedure TPatternEdit.SelectBoxChange(Sender: TObject);
begin
  ReadSet(SelectBox.ItemIndex);
end;

procedure TPatternEdit.ReadSet(Index: Integer);
var I: Integer;
begin
  SelectBox.Items.BeginUpdate;
  InputBox.Items.BeginUpdate;
  OutputBox.Items.BeginUpdate;
  try
    SelectBox.Items.Clear;
    InputBox.Items.Clear;
    OutputBox.Items.Clear;
    with PatternSet do begin
      Edit2.Text := IntToStr(PatternNo);
      if PatternNo > 0 then begin
        if Index > PatternNo - 1 then Index := 0;
        for I := 0 to (PatternNo - 1) do
          SelectBox.Items.Add(IntToStr(I));
        SelectBox.ItemIndex := Index;
        for I := 0 to (InputNo - 1) do
          InputBox.Items.Add(FloatToStrF(Items[Index].Inputs[I], ffExponent, 7, 2));
        for I := 0 to (OutputNo - 1) do
          OutputBox.Items.Add(FloatToStrF(Items[Index].Outputs[I], ffExponent, 7, 2));
        DeleteAllBtn.Enabled := True;
        DeleteBtn.Enabled := True;
        InsertBtn.Enabled := True;
      end
      else begin
        DeleteAllBtn.Enabled := False;
        DeleteBtn.Enabled := False;
        InsertBtn.Enabled := False;
        for I := 0 to (InputNo - 1) do InputBox.Items.Add('');
        for I := 0 to (OutputNo - 1) do OutputBox.Items.Add('');
      end;
      InputSpin.Value := InputNo;
      OutputSpin.Value := OutputNo;
    end;
  finally
    SelectBox.Items.EndUpdate;
    InputBox.Items.EndUpdate;
    OutputBox.Items.EndUpdate;
  end;
  InputBoxClick(Self);
end;

end.



